#!/usr/bin/php
<?php
$_SERVER['DOCUMENT_ROOT']='/usr/local/esia/public_html/';
include_once '/usr/local/esia/public_html/esia.conf.php';
include_once '/usr/local/esia/public_html/plugin/backup/inc/Model_Backup.php';

$dbconnection = new PDO('pgsql:host='.DB_HOST.';port='.DB_PORT.';dbname='.DB_NAME, DB_USERNAME, DB_PWD);
$dbconnection->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_WARNING);

//$iplong = ip2long("10.7.0.7");

//ESIA PLUGIN LIB
$STR_STATUS=array("OK","WARNING","CRITICAL","UNKNOWN");
$INT_STATUS=array_flip($STR_STATUS);
$PRIORITIES_STATES=array("CRITICAL","WARNING","UNKNOWN","OK");

function print_usage($bad_usage){
	echo 'USAGE : '.basename($_SERVER['PHP_SELF'])." -H HOST -n NAME [OPTIONS]\n";
	if($bad_usage) echo 'HELP  : '.basename($_SERVER['PHP_SELF'])." -h\n";
}

function print_help(){
	print_usage(false);
	echo "\n";
	echo "\t-H, ".str_pad("--host=HOST",20)		."IP address of backup host\n";
	echo "\t-I, ".str_pad("--ignore=INTEGER",20)	."Ignore INTEGER backups errors\n";
	echo "\t-J, ".str_pad("--job",20)		."Mode job\n";
	echo "\t-R, ".str_pad("--regex",20)		."Enable regex mode for job/source name\n";
	echo "\t-n, ".str_pad("--name=NAME",20)         ."Name of the object (VM, server, data,...) to verify backup\n";
	echo "\t-w, ".str_pad("--warning=WARN",20)	."Default : 1. Check return warning if success backup number is less than or equal to WARN\n";
	echo "\t-c, ".str_pad("--critical=CRITIC",20)	."Default : 0. Check return critical if success backup number is less than or equal to CRITIC\n";
	echo "\t-p, ".str_pad("--period=PERIOD",20)	."Default : 1d. PERIOD in X[h|w|d] (48h, 2d, 1w). h for hours, d for days, w for weeks\n";
	echo "\n";
	echo "\t-h, ".str_pad("--help",20)		."Print this help\n";
	echo "\n";
}

function getParam($options,$name1,$name2,$default=null){
	if( isset( $options[$name1] ) ) return $options[$name1];
	if( isset( $options[$name2] ) ) return $options[$name2];
	return $default;
}

function exit_state($state,$msg=null,$usage=false){
	$STR_STATUS=$GLOBALS["STR_STATUS"];
	$INT_STATUS=$GLOBALS["INT_STATUS"];

	$ret=$state;
	if( is_string($state) ) $ret=$INT_STATUS[strtoupper($state)];
	else $state=$STR_STATUS[$ret];

	if( $msg != null ) echo "$state - $msg\n";
	if( $usage ) print_usage(true);
	exit($ret);
}

function calcul_state($a,$b=null){
	$STR_STATUS=$GLOBALS["STR_STATUS"];
	$INT_STATUS=$GLOBALS["INT_STATUS"];
	$PRIORITIES_STATES=$GLOBALS["PRIORITIES_STATES"];

	if(is_string($a) ) $a=$INT_STATUS[strtoupper($a)];
	if(is_string($b) ) $b=$INT_STATUS[strtoupper($b)];

	foreach($PRIORITIES_STATES as $value){
		$value = $INT_STATUS[$value];

		if( $value == $a || $value == $b ) return $value;
	}
	return $INT_STATUS["UNKNOWN"];
}

//Recuperation des infos
function parseQuery($query){
	$ret = array();
	while($row=$query->fetch(PDO::FETCH_OBJ)){
		$duration = null;
		$job_end = null;

		$dt = new DateTime($row->job_start_time);
		$job_start = $dt->format('U');

		if($row->job_end_time != null) {
			$dt = new DateTime($row->job_end_time);
			$job_end = $dt->format('U');
			$duration = $job_end - $job_start;
		}

		$ret[] = array(
			'start' => $job_start,
			'end'   => $job_end,
			'size'  => $row->job_size,
			'duration' => $duration,
			'type'	   => $row->job_type,
			'name'	   => $row->job_name,
			'source'   => $row->job_source,
			'storage'  => $row->job_storage,
			'status_code' => $row->job_status,
			'status_desc' => $row->job_messages,
		);
	}
	return $ret;
}

function reqByPeriod($since=null,$last=null,$group=null,$name=null,$regex_mode=false){
	global $dbconnection, $server_id,$type;

	if( $since == null ) $since = 1;
	if( $last == null) $last = time();

	if( $since > $last ) throw new Exception();
	$sql = 'SELECT job_id,job_name,job_source,job_storage,job_status,job_messages,job_start_time,job_end_time,job_size,job_type FROM backup_job';
	$sql.= ' WHERE job_server_id = :srv_id AND job_start_time BETWEEN to_timestamp(:since) AND to_timestamp(:last)';

	$params = array(
		':since'=>$since,
		':last' =>$last,
		':srv_id'=>$server_id
	);

	$error = true;
	if( is_string($group) && is_string($name)){
		if( $regex_mode){
			$sql.= " AND $group ~ :group";
		}
		else{
			$sql.= " AND $group = :group";
		}
		$params[':group'] = $name;
		$error = false;
	}
	else if ( $group == null || $name == null ){
		$error = false;
	}

	if( !empty($type) ) {
		$sql.= " AND job_type = :type";
		$params[':type'] = $type;
	}


	if( $error ) throw new Exception();

	$sql.= ' ORDER BY COALESCE(job_end_time,job_start_time) DESC, job_status ASC, job_id DESC';

	$query = $dbconnection->prepare($sql);
	$query = Model_esia::executeStatement($query,$params);

	return parseQuery($query);
}

function getPeriodSources($source,$since=null,$last=null,$regex_mode=false){
	return reqByPeriod($since,$last,'job_source',$source,$regex_mode);
}

function getPeriodJobs($job,$since=null,$last=null,$regex_mode=false){
	return reqByPeriod($since,$last,'job_name',$job,$regex_mode);
}

function parseDate($start){
	$start = strtolower($start);
	if(preg_match('/^[-]?(\d+)([hdwmy]?)$/',$start,$matches)){
		if(!empty($matches[2])){
			$matches[2]=strtoupper($matches[2]);
			switch($matches[2]){
			case 'H' :
				$since = time() - $matches[1] * 3600;
				break;
			case 'D' :
				$since = time() - $matches[1] * 86400;
				break;
			case 'W' :
				$since = time() - $matches[1] * 604800;
				break;
			case 'M' :
				$date = new DateTime();
				$date->sub(new DateInterval('P'.$matches[1].$matches[2]));
				$since=$date->format('U');
				break;
			case 'Y' :
				$date = new DateTime();
				$date->sub(new DateInterval('P'.$matches[1].$matches[2]));
				$since=$date->format('U');
				break;
			}
		}
		else $since = $matches[1];
	}
	else throw new Exception();
	return $since;
}

$SHORT_OPTS="H:I:JRn:i:w:c:p:t:h";
$LONG_OPTS=array("host:","id:","name:","ignore:","job","regex","warn:","critic:","period:","help");

$options = getopt($SHORT_OPTS, $LONG_OPTS);

$host = getParam($options,'H','host');
$host = getParam($options,'I','id',$host);
$name = getParam($options,'n','name');
$ignore = getParam($options,'i','ignore',0);
$warn = getParam($options,'w','warning',1);
$critic = getParam($options,'c','critic',0);
$period = getParam($options,'p','period','1d');

if( $host == NULL || $name == NULL) exit_state(3,'You  forget parameters -H (host) or -n (name)');
if( !is_numeric( $warn) || !is_numeric( $critic) ) exit_state(3,'-w (warning) and -c (critical) parameters must be an integer');
if( $warn < $critic ) exit_state(3,'The warning value must be greater or equal to critical value');

$job_mode = ( isset($options['J']) || isset($options['job']));
$regex_mode = ( isset($options['R']) || isset($options['regex']));

try{
	$ts_period = parseDate($period);
}
catch(Exception $ex){
	exit_state(3,"Unable to parse the period -p (period) $period");
}

//Récupération du device_id
$server_id = NULL;
if(is_numeric($host)){
	$sql = "SELECT device_id FROM esia_device WHERE device_type = 'backup server' AND device_id=?";
	$sqlParam = $host;
}
else{
	$sql = "SELECT device_id FROM esia_device WHERE device_type = 'backup server' AND device_ip_addr=?";
	$sqlParam = ip2long($host);
}
try{
	$query = $dbconnection->prepare($sql);
	$query = Model_esia::executeStatement($query,array($sqlParam));
	$row=$query->fetch(PDO::FETCH_OBJ);
	if(is_object($row)){
		$server_id = $row->device_id;
	}
}
catch(Exception $ex){}
if( $server_id == NULL ) exit_state(3,"Unable to found $host device");

$now_ts = time();
if(! $job_mode){
	$data = getPeriodSources($name,$ts_period,$now_ts,$regex_mode);
}
else{
	$data =  getPeriodJobs($name,$ts_period,$now_ts,$regex_mode);
}
$bck_states=array(
	'total' => 0,
	'last' => 0,
	'success' => 0,
	'warn' => 0,
	'critic' => 0,
);

usort($data, function($a, $b) {
    return $b['start'] <=> $a['start']; //Reverse sort (b avant a)
});

$errors_founds=false;
$o_ignore = $ignore;
foreach($data as $bck){
	$bck_states['total']++;
	if( $bck['status_code'] == 0 ){
		$bck_states['success']++;
		if(!$errors_founds) {
			$bck_states['last']++;
		}
	}
	else{
		if( $ignore > 0 )	$ignore--;
		else 			$errors_founds=true;
		if( $bck['status_code'] == 1 ){
			$bck_states['warn']++;
		}
		elseif( $bck['status_code'] == 2 ){
			$bck_states['critic']++;
		}
	}
}

//Calcul des complete_jobs
if( $job_mode ){
	$jobs_arr = array();
	foreach( $data as $bck){
		if(!isset($jobs_arr[$bck['source']])){
			$jobs_arr[$bck['source']] = 0;
		}
		if($bck['status_code']==0){
			$jobs_arr[$bck['source']]++;
		}
	}

	$min = 0;
	$max = 0;
	$sources_arr = array();
	foreach($jobs_arr as $key => $value){
		if( $min == 0  || $value < $min ){
			$min = $value;
			$sources_arr = array();
		}
		if( $value == $min ){
			$sources_arr[] = $key;
		}
		if( $max < $value ){
			$max = $value;
		}
	}
	$showed_min = $min;
	if( $min == $max ){
		$sources_arr = array();
	}
	else{
		$min = $min + $o_ignore;
		if( $min > $max ){
			$min = $max;
		}
	}

	$bck_states['last'] = $min;
}

if( $bck_states['last'] <= $critic ){
	$state=2;
}
elseif( $bck_states['last'] <= $warn ){
	$state=1;
}
else{
	$state=0;
}

$showed_since = date('d-m-Y H:i:s',$ts_period);
$showed_end = date('H:i:s',$now_ts);

if( !$job_mode ){
	$message = sprintf('%d last backups OK. Last %s period, between now(%s) to %s, %d backups started (%d success, %d with warnings and %d with errors, ignore %d errors)',
		$bck_states['last'],$period,$showed_end,$showed_since,$bck_states['total'],$bck_states['success'],$bck_states['warn'],$bck_states['critic'],$o_ignore
	);
}
else{
	$message = sprintf('%d Completed jobs OK (%d backups for sources:%s). Last %s period, between now(%s) to %s, %d backups started (%d success, %d with warnings and %d with errors, ignore %d errors/missing)',
		$bck_states['last'],$showed_min,implode(', ',$sources_arr),$period,$showed_end,$showed_since,$bck_states['total'],$bck_states['success'],$bck_states['warn'],$bck_states['critic'],$o_ignore
	);
}

exit_state($state,$message);
?>
