#!/usr/bin/php
<?php
define('ECATP_PATH','/usr/local/esia/plugins/check_gesa');
define('PLUGIN_BIN',basename($argv[0]));
define('DRIVER','/usr/local/esia/plugins/lib/bareosdrv.py');
$JOBSTATUS = array(
		'C' => array( 'status' => 4 , 'desc' => 'Created, not yet running' ),
		'R' => array( 'status' => 4 , 'desc' => 'Running' ),
		'B' => array( 'status' => 1 , 'desc' => 'Blocked' ),
		'T' => array( 'status' => 0 , 'desc' => 'Completed successfully' ),
		'E' => array( 'status' => 2 , 'desc' => 'Terminated with errors' ),
		'e' => array( 'status' => 1 , 'desc' => 'Non-fatal error' ),
		'f' => array( 'status' => 2 , 'desc' => 'Fatal error' ),
		'D' => array( 'status' => 4 , 'desc' => 'Verify found differences' ),
		'A' => array( 'status' => 3 , 'desc' => 'Canceled by user' ),
		'I' => array( 'status' => 2 , 'desc' => 'Incomplete job' ),
		'L' => array( 'status' => 4 , 'desc' => 'Committing data' ),
		'W' => array( 'status' => 1 , 'desc' => 'Terminated with warnings' ),
		'l' => array( 'status' => 4 , 'desc' => 'Doing data despooling' ),
		'q' => array( 'status' => 4 , 'desc' => 'Queued waiting for device' ),
		'F' => array( 'status' => 4 , 'desc' => 'Waiting for Client' ),
		'S' => array( 'status' => 4 , 'desc' => 'Waiting for Storage daemon' ),
		'm' => array( 'status' => 4 , 'desc' => 'Waiting for new media' ),
		'M' => array( 'status' => 4 , 'desc' => 'Waiting for media mount' ),
		's' => array( 'status' => 4 , 'desc' => 'Waiting for storage resource' ),
		'j' => array( 'status' => 4 , 'desc' => 'Waiting for job resource' ),
		'c' => array( 'status' => 4 , 'desc' => 'Waiting for client resource' ),
		'd' => array( 'status' => 4 , 'desc' => 'Waiting on maximum jobs' ),
		't' => array( 'status' => 4 , 'desc' => 'Waiting on start time' ),
		'p' => array( 'status' => 4 , 'desc' => 'Waiting on higher priority jobs' ),
		'i' => array( 'status' => 4 , 'desc' => 'Doing batch insert file records' ),
		'a' => array( 'status' => 4 , 'desc' => 'SD despooling attributes' ),
);

$options = getopt('A:M:O:');

$EXIT_SUCCESS=0;
$EXIT_NOT_IMPLEMENTED=-1;
$EXIT_ERROR=1;

if( !isset($options['M'])){
	echo "Parameter -M is missing\n"; exit($EXIT_ERROR);
}

$METHOD=$options['M'];
if(isset($options['O'])){
	if(is_array($options['O'])){
		$OPTIONS=$options['O'];
	}
	else{
		$OPTIONS=array($options['O']);
	}
}
else{
	$OPTIONS=array();
}

$action = strtolower($METHOD);

$AUTH_FILE=$options['A'];
if(is_readable($AUTH_FILE)){
        $options = json_decode(file_get_contents($AUTH_FILE),true);
}

foreach($OPTIONS as $opt){
	$exp=explode('=',$opt);
	$exp[0] = trim($exp[0]);
	$exp[1] = trim($exp[1]);

	if(count($exp) != 2 ) {
		echo "Incorrect Option -O format ($opt) must have format -O <name>=<value> -O <name>=<value> ... \n"; exit($EXIT_ERROR);
	}
	if( isset($options[$exp[0]]) ){
		echo "Option -O $opt overwrite another options parameter or auth file option\n"; exit($EXIT_ERROR);
	}

	$options[$exp[0]]=$exp[1];
}

if( empty($options['host']) ){
        echo "JSON file $AUTH_FILE hasn't required 'host' parameter\n"; exit($EXIT_ERROR);
}
if( empty($options['pass']) ){
        echo "JSON file $AUTH_FILE hasn't required 'pass' parameter\n"; exit($EXIT_ERROR);
}

//Si on à l'options gesasn, c'est qu'on doit passer à travers une unity
if( isset($options['gesasn'])){
	$gesasn = $options['gesasn'];
	unset($options['gesasn']);

	$parameters = "-M $action";
	foreach($options as $key => $value){
		$parameters.= " -O $key=$value";
	}

	$cmd = ECATP_PATH . " -G $gesasn -C ". PLUGIN_BIN . ' -A ' ."'$parameters'";
	exec($cmd,$output,$ret);
	echo implode("\n",$output);
	exit($ret);
}

switch($action){
	case 'getgroupsids' :
		$ret = getGroups($result,$options);
		break;
	case 'getjobsids' :
		$ret = getJobs($result,$options);
		break;
	case 'getjobsvalues' :
		$ret = getJobsValues($result,$options,true);
		break;
	case 'getnodesvalues' :
		$ret = getNodesValues($result,$options);
		break;
	case 'getmessagesvalues' :
		$ret = getMessagesValues($result,$options);
		break;
	default :
		echo "Method $action not implemented\n"; exit($EXIT_NOT_IMPLEMENTED);
}

echo json_encode($result,JSON_PRETTY_PRINT)."\n";
exit($ret);

function getGroups(&$result,$options){
	$output=run($options['host'],$options['pass'],'show jobs',false);
	$groups=parseConfig($output);

	$result=array();
	$ids = array();
	foreach($groups as $group){
		$ids[]=$group['Name'];
	}
	sort($ids);
	$result['ids']=$ids;
	return $GLOBALS['EXIT_SUCCESS'];
}

function getJobsByStatus($cmd,$options,$status){
	if( $status == NULL ){
		return run($options['host'],$options['pass'],$cmd,true)['jobs'];
	}

	$jobs = array();
	foreach($GLOBALS['JOBSTATUS'] as $k => $js){
		foreach($status as $st){
			if( $st == $js['status'] ){
				$result = run($options['host'],$options['pass'],"$cmd jobstatus=$k",true)['jobs'];
				$jobs = array_merge($jobs,$result);
			}
		}
	}
	return $jobs;
}

function getJobs(&$result,$options){
	$hours = null;
	$groups = null;
	$except = array();
	$status = null;

	$cmd = 'list jobs';

	if(isset($options['period'])){
		$hours = parseDate($options['period']);
		$cmd.= " hours=$hours";
	}
	if( isset($options['status']) && strtolower($options['status']) != 'all'){
		$status = explode(',',$options['status']);
		foreach($status as $st){
			if( !is_numeric($st) || $st < 0 || $st > 4 ){
				echo "$status must be between 0 and 4\n";
				exit($GLOBALS['EXIT_ERROR']);
			}
		}
	}
	if(isset($options['group']) && strtolower($options['group']) != 'all' ){
		$groups = explode(',',$options['group']);
		$jobs = array();
		foreach($groups as $group){
			$jobs = array_merge($jobs,getJobsByStatus("$cmd job=".'"'.$group.'"',$options,$status));
		}
	}
	else{
		$jobs = getJobsByStatus($cmd,$options,$status);
	}
	if( isset($options['except'] )){
		$except = explode(',',$options['except']);
	}
	else $except = array();

	$ids = array();
	foreach($jobs as $job){
		if( !in_array($job['jobid'],$except) ){
			$ids[] = $job['jobid'];
		}
	}
	sort($ids);
	$result['ids']=$ids;
	return $GLOBALS['EXIT_SUCCESS'];
}

function messageToArray($message){
	$arr = array();
	$tmp = explode("\n",$message['logtext']);
	$first = array_shift($tmp);
	foreach($tmp as $line){
		$exp = explode(':',$line);
		if( isset($exp[1]) ){
			$key = trim($exp[0]);
			$value = trim($exp[1]);
			$arr[$key]=$value;
		}
	}
	if(empty($arr) ) return false;
	if(!isset($arr['Termination'])) return false;

	if(preg_match('/(\S+)\s+\w+\s+\d+\s*:/',$first,$matches)){
		$arr['server_name'] = $matches[1];
	}
	return $arr;
}


function getJobsValues(&$result,$options){
	$jobsids = array();
	if( isset($options['ids']) ){
		$jobsids = explode(',',$options['ids']);
	}
	if( empty($jobsids) ) {
		echo "Parameter ids is missing\n";
		exit($GLOBALS['EXIT_ERROR']);
	}

	$server = run($options['host'],$options['pass'],"version",false)[0];
	$server = explode(' ',$server)[0];

	$result = array();
	foreach($jobsids as $id){
		$job = run($options['host'],$options['pass'],"list jobid=$id",true)['jobs'][0];
		global $JOBSTATUS;
		$letter = $job['jobstatus'];
		$status_esia = $JOBSTATUS[$letter]['status'];
		$status_backup = $JOBSTATUS[$letter]['desc'];
		$end_time = null;
		$storage = null;
		if($status_esia != 4 ){
			$messages = run($options['host'],$options['pass'],"list joblog jobid=$id",true)['joblog'];
			$last = array_pop($messages);
			$end_time = $last['time'];
			$storage = explode('(',messageToArray($last)['Storage'])[0];
			$storage = trim($storage,' "');
		}

		$parse['intern_id'] = $job['jobid'];
		$parse['name'] = $job['name'];
		$parse['server'] = $server;
		$parse['source'] = $job['client'];
		$parse['storage'] = $storage;
		$parse['size'] = $job['jobbytes'];
		switch($job['level']){
			case 'F' :
				$parse['type'] = 'Full';
				break;
			case 'D' :
				$parse['type'] = 'Differential';
				break;
			case 'I' :
				$parse['type'] = 'Incremental';
				break;
			default :
				$parse['type'] = null;
		}
		global $JOBSTATUS;
		$letter = $job['jobstatus'];
		$parse['status_esia'] = $status_esia;
		$parse['status_backup'] = $status_backup;
		$parse['start_time'] = date_format(new DateTime($job['starttime']),'U');
		$parse['end_time'] = date_format(new DateTime($end_time),'U');
		$result[] = $parse;
	}
	return $GLOBALS['EXIT_SUCCESS'];
}



function getNodesValues(&$result,$options){
	$result = array();

	$output = run($options['host'],$options['pass'],'show clients',false);
	$nodes = parseconfig($output);
	foreach($nodes as $node){
		$result['sources'][$node['Name']] = array('address'=>$node['Address']);
	}
	$output = run($options['host'],$options['pass'],'show storages',false);
	$nodes = parseconfig($output);
	foreach($nodes as $node){
		if( $node['TlsVerifyPeer'] != 'yes' ) $peer = '(VerifyPeer = No)';
		else $peer = '';

		if( $node['TlsRequire'] == 'yes' ) $tls = "Required $peer";
		else if( $node['TlsEnable'] == 'yes' ) $tls = "Enabled $peer";
		else $tls = 'None';

		$result['storages'][$node['Name']] = array(
				'address'=>$node['Address'],
				'port'=>$node['SddPort'],
				'tls'=>$tls,
		);
	}

	return $GLOBALS['EXIT_SUCCESS'];
}

function getMessagesValues(&$result,$options){
	$jobsids = array();
	if( isset($options['ids']) ){
		$jobsids = explode(',',$options['ids']);
	}
	if( empty($jobsids) ) {
		echo "Parameter ids is missing\n";
		exit($GLOBALS['EXIT_ERROR']);
	}

	foreach($jobsids as $id){
		$messages = run($options['host'],$options['pass'],"list joblog jobid=$id",true)['joblog'];
		$parse['intern_id'] = $id;
		$parse['messages'] = array();
		foreach($messages as $message){
			$m['time'] = date_format(new DateTime($message['time']),'U');
			$m['text'] = $message['logtext'];
			$parse['messages'][] = $m;
		}
		$result[]=$parse;
	}
	return $GLOBALS['EXIT_SUCCESS'];
}

function run($host,$pass,$command,$json=false){
	$cmd = DRIVER." -a $host -P $pass -c '$command' 2>/dev/null";
	if( $json ) $cmd = "$cmd -j";

	exec($cmd,$output,$ret);
	if( $ret != 0 ) {
		echo "Driver ERROR\n";
		exit($GLOBALS['EXIT_ERROR']);
	}

	if( $json ) return json_decode($output[0],true);
	return $output;
}
function parseConfig($lines){
	$items =array();
	$parse = implode('|',$lines);
	if( !preg_match_all('/(\w+)\s*\|*{([^}]*)/',$parse,$matches)) return array();

	foreach($matches[2] as $match){
		$item = array();
		if( preg_match_all('/([^|=]*)=([^|]*)/',$match,$matches2)){
			for($i=0;$i<count($matches2[1]);$i++){
				$key = trim($matches2[1][$i]);
				$value = trim(trim($matches2[2][$i]),'"');
				$item[$key]=$value;
			}
			$items[]=$item;
		}
	}
	return $items;
}

function parseDate($start){
	$start = strtolower($start);
	if(preg_match('/^[-]?(\d+)([hdwmy]?)$/',$start,$matches)){
		if(!empty($matches[2])){
			$matches[2]=strtoupper($matches[2]);
			switch($matches[2]){
				case 'H' :
					$since = time() - $matches[1] * 3600;
					break;
				case 'D' :
					$since = time() - $matches[1] * 86400;
					break;
				case 'W' :
					$since = time() - $matches[1] * 604800;
					break;
				case 'M' :
					$date = new DateTime();
					$date->sub(new DateInterval('P'.$matches[1].$matches[2]));
					$since=$date->format('U');
					break;
				case 'Y' :
					$date = new DateTime();
					$date->sub(new DateInterval('P'.$matches[1].$matches[2]));
					$since=$date->format('U');
					break;
			}
		}
		else $since = $matches[1];
	}
	else throw new Exception();

	$hours = ( time() - $since ) / 3600;
	return intval($hours);
}

function getJobsMessages($id){
	if(is_array($id)){
		$arr = array();
		foreach($id as $value){
			$job = getJobsMessages($value);
			foreach($job as $key => $content){
				$arr[$key] = $content;
			}
		}
		return $arr;
	}
	else {
		$tmp = run("list joblog jobid=$id",true);
		if( !isset($tmp['joblog'])) return array( $id => FALSE );
		$joblog = $tmp['joblog'];
		return array( $id => $joblog );
	}
}
?>
