#!/usr/bin/php
<?php
include "eurobackup.class.php";

$timeout = 900;

//Armement timeout
pcntl_alarm($timeout+2);
pcntl_signal(SIGALRM, "sig_handler");

$EXIT_SUCCESS=0;
$EXIT_NOT_IMPLEMENTED=-1;
$EXIT_ERROR=1;

$parameters = getopt('A:M:O:');

if(!isset($parameters['M']))
{
    echo "Missing required parameter Method (-M)\n";
    exit($EXIT_ERROR);
}
$action = strtolower($parameters['M']);

$options = array();
if( isset($parameters['A']))
{
    $AUTH_FILE=$parameters['A'];
    if(!is_readable($AUTH_FILE))
    {
        echo "File $AUTH_FILE missing or not readable\n"; 
        exit($EXIT_ERROR);
    }
    $options = json_decode(file_get_contents($AUTH_FILE),true);

    if( !is_array($options))
    {
        echo "Unable to decode auth JSON file $AUTH_FILE\n";
        exit($EXIT_ERROR);
    }
}

if( isset($parameters['O']) )
{
    if( !is_array($parameters['O']) )
    {
        $parameters['O'] = array($parameters['O']);
    }
}

$required_options = 'SysUser,SysPwd,LoginName,Password';
$required_options = explode(',',$required_options);
foreach($parameters['O'] as $opt_val)
{
    $exp=explode('=',$opt_val);        
    $exp[0] = trim($exp[0]);
    $exp[1] = trim($exp[1]);

    $options[$exp[0]] = $exp[1];
}

foreach($required_options as $req)
{
    if( empty($options[$req]) )
    {
        echo "JSON file $AUTH_FILE or command line options hasn't required '$req' parameter\n"; exit($EXIT_ERROR);
    }
}

//$options['ids']=explode(',',$options['ids']);

$api = new EurobackupApi();
$ret = $api->authentication($options["url"],$options);

//$options["BackupSetID"]="1518451697931";
//$ret = $api->getListBackupSets($options["url"],$options);

$data = array();

if($ret==true)
{    
    switch($action)
    {
        case 'getjobsids' :        	
		$ret = array();
                $ret['ids'] = getJobsIDs($api,$options["url"],$options);
                break;
        case 'getjobsvalues' :               
                $ret = getJobsInfos($api,$options["url"],$options);
                break;
        default :
                echo "Method $action not implemented\n"; exit($EXIT_NOT_IMPLEMENTED);        
    }
    echo json_encode($ret,JSON_PRETTY_PRINT)."\n";
    exit($EXIT_SUCCESS);    
}
else
{
    echo "Error authentication, verify login/pwd \n";
    exit($EXIT_ERROR);
}

function getJobsIDs($api,$url,$options){
        $period_ts = period_to_ts($options["period"]);
        $all_dates = all_days_since_ts($period_ts,'Y-m-d');

        $list_eurobackup_jobs = array();
        foreach($all_dates as $date){
                $options["BackupDate"]=$date;
                $backups = $api->getListBackupJobStatus($url,$options);
                if( is_array($backups)){
                        $list_eurobackup_jobs = array_merge($list_eurobackup_jobs,$backups);
                }
        }
	
/*	echo "NB : " .count($list_eurobackup_jobs)."\n";
	$list_eurobackup_list = array();
	foreach($list_eurobackup_jobs as $job){
		
	}
*/
        $esia_backups_list = array();
        foreach($list_eurobackup_jobs as $eurobackup){
                $backup = eurobackupAPI_to_esiaAPI($eurobackup);
                if( $backup['status_backup'] == 'BS_BACKUP_NOT_FINISHED' || (empty($backup['end_time']) && date_between($backup['start_time']+86400,$period_ts,time())) ||  (date_between($backup['end_time'],$period_ts,time()))){
                        if( !isset($options['status']) || $backup['status_esia'] == $options['status'] ){
                                $esia_backups_list[] = $backup['intern_id'];
                        }
                }
        }
        return $esia_backups_list;
}

function getJobsInfos($api,$url,$options){
	$options['ids']=explode(',',$options['ids']);

	$id_by_keys = array();
	$day_list = array();
	$set_list = array();
	foreach( $options['ids'] as $id ){
		$day = id_to_day($id);		
		$day_list[$day] = true;
		$id_by_keys[$id] = true;
		$set_list[explode('_',$id)[0]]=true;
	}	
	$day_list = array_keys($day_list);
	
	foreach($set_list as $key => $value){
		$data = array("SysUser"=>$options["SysUser"],
                	"SysPwd"=>$options["SysPwd"],
                        "LoginName"=>$options["LoginName"],
                        "BackupSetID"=>$key);

		$ret = $api->getGetbackupSets($options["url"],$data);
		
		$type=null;
		if($ret["DeltaType"]=="I")
                {
                	$type="Incremental";
                }
                elseif($ret["DeltaType"]=="D")
                {
                	$type="Differential";
                }	

		$source=$ret["SelectedSourceList"];
/*
		if(!empty($ret["SelectedSourceList"]) && isset($ret["SelectedSourceList"])){
			$source=array();
			foreach ($ret["SelectedSourceList"] as $s)
                        {
                        	$source[] = $s;
                        }
                        $source = implode(',',$source);
		}
*/
		$set_list[$key] = array('type' => $type, 'source' => $source);
	}

	$eurobackuplist = array();
	foreach($day_list as $day){
		$options["BackupDate"]=$day;
		$backups = $api->getListBackupJobStatus($url,$options);
		$eurobackuplist = array_merge($eurobackuplist,$backups);
	}

	$esia_backups_list = array();
	foreach($eurobackuplist as $eurobackup){
		$backup = eurobackupAPI_to_esiaAPI($eurobackup);
		if( isset( $id_by_keys[$backup['intern_id']] )){
			$backup_set = $set_list[explode('_',$backup['intern_id'])[0]];
			$backup['type'] = $backup_set['type'];
			$backup['source']= $backup_set['source'];
			if( empty($backup['source']) ){
				$backup['source'] = $backup['name'];
				$backup['intern_id'] = 'NB'.$backup['intern_id'];
				$esia_backups_list[] = $backup;
			}
			else{
				$src_list = $backup['source'];
				sort($src_list);
				$intern_id = $backup['intern_id'];
				foreach( $src_list as $key => $src ){
					if( preg_match( '/^.*[Dd]atacenter[^\/]*\/.*[Dd]atacenter[^\/]*(\/\d+-[^\/]*){2}\/(\d+(-[0-f]+){5})/',$src,$matches) ){
						if( !empty($matches[2])){
							$src = $matches[2];
						}
					}

					$tmp = $backup;	
					$tmp['intern_id'] = $key.'B'.$intern_id;
					$tmp['source'] = $src;
					$esia_backups_list[] = $tmp;
				}
			}
		}
	}
	return $esia_backups_list;
}

function id_to_day($id){
        $ds = explode('_',$id)[2];
        $date = DateTime::createFromFormat('Y-m-d-H-i-s', $ds);
        if( $date === false ) return false;
        return $date->format('Y-m-d');
}

function period_to_ts($start){
        $start = strtolower($start);
        if(preg_match('/^[-]?(\d+)([hdwmy]?)$/',$start,$matches)){
                if(!empty($matches[2])){
                        $matches[2]=strtoupper($matches[2]);
                        switch($matches[2]){
                                case 'H' :
                                        $since = time() - $matches[1] * 3600;
                                        break;
                                case 'D' :
                                        $since = time() - $matches[1] * 86400;
                                        break;
                                case 'W' :
                                        $since = time() - $matches[1] * 604800;
                                        break;
                                case 'M' :
                                        $date = new DateTime();
                                        $date->sub(new DateInterval('P'.$matches[1].$matches[2]));
                                        $since=$date->format('U');
                                        break;
                                case 'Y' :
                                        $date = new DateTime();
                                        $date->sub(new DateInterval('P'.$matches[1].$matches[2]));
                                        $since=$date->format('U');
                                        break;
                        }
                }
                else $since = $matches[1];
        }
        else throw new Exception();

        return date('U',$since);
}

function all_days_since_ts($ts,$format){
        $dates = array();
        $last = time();
        while( $ts <= $last ) {
                $dates[] = date($format, $ts);
                $ts = strtotime('+1 days', $ts);
        }
        return $dates;
}

function date_between($date,$since,$last=null){
        if(!is_numeric($date) ){
                $date = eurobackup_datetime_to_ts($date);
        }
        if(!is_numeric($since) ){
                $since = eurobackup_datetime_to_ts($since);
        }
        if( $last == null ){
                $last=time();
        }
        else if(!is_numeric($last)){
                $last = eurobackup_datetime_to_ts($last);
        }

        if( $date === false || $since == false || $last == false) return false;
        if( $date < $since ) return false;
        if( $date > $last  ) return false;
        return true;
}

function eurobackup_datetime_to_ts($eurobackup){
        $eurobackup = str_replace(array(':',' '),'-',$eurobackup);
        $date = DateTime::createFromFormat('Y-m-d-H-i-s', $eurobackup);
        if( $date === false ){
                $date = DateTime::createFromFormat('Y-m-d', $eurobackup);
        }
	if( $date === FALSE ) return '';
        return $date->format('U');
}

function eurobackupAPI_to_esiaAPI($backup){
        $esia['intern_id']      = $backup['BackupSetID'] . '_' .  $backup['DestinationID'] . '_' . $backup['ID'];
        $esia['name']           = $backup['BackupSetName'];
        $esia['status_backup']  = $backup['BackupJobStatus'];
        if( $esia['status_backup'] == 'BS_STOP_SUCCESS' ){
                $esia['status_esia'] = 0;
        }
        else{
                $esia['status_esia'] = 2;
        }

        $esia['start_time']     = eurobackup_datetime_to_ts($backup['StartTime']);
        $esia['end_time']       = eurobackup_datetime_to_ts($backup['EndTime']);

        $esia['size']           = $backup['UploadSize'];
        $esia['type']           = null;
        $esia['source']         = 'none';
        return $esia;
}

declare(ticks = 1);
function sig_handler($signo){
	global $EXIT_ERROR;
	exit($EXIT_ERROR);
}
?>
