<?php

class EurobackupApi
{	
	private $id;
	private $host;
	private $user_api;	
	private $password_api;
	private $user;
	private $password;
		
	public function __construct() {}
	
	/**
	 * Effectue une requete get sur le satellite
	 * @param string $path url du serveur
	 * @param string $param (optionnel) parametre supplementaire dans le cas d'un filter
	 * @retval si requete OK return json. Si erreur return null
	 */
	public function get($url,$param)
	{				
		$options = array(
  			'http' => array(
    		'method'  => 'POST',
    		'content' => json_encode($param),
    		'header'=>  "Content-Type: application/json\r\n".
               			"Accept: application/json\r\n"
    		)
		);

		$context  = stream_context_create($options);
		$result = file_get_contents($url,false,$context);
		$data = json_decode($result,true);
		return $data;
	}

	/**
	 * verifyData
	 * @param array $data
	 * @retval si OK return true	 
	 */
	public function verifyData($data)
	{			
		if($data['Status']=="OK")
		{
			if(isset($data["Data"]))
				return $data["Data"];
			elseif(isset($data["BkpStatus"]))
				return $data["BkpStatus"];
			elseif(isset($data["RestoreStatus"]))
				return $data["RestoreStatus"];
		}
		else
		{
			echo "ERROR => ".$data['Message']." \n";
			echo "ERROR => ".$data['ExptType']." \n";
			return null;
		}
		return null;
	}

	/**
	 * Authentification du client sur base de son username et son hash
	 * @param string $url url du serveur
	 * @param string $param array contenant (login,pwd,...)	 
	 * @retval si authentification OK return true	 
	 */
	public function authentication($url,$param)
	{	
		$data = self::get($url."/obs/api/json/AuthUser.do",$param);
		
		if($data['Status']=="OK") // Client Authentifié
		{
			$this->user_api = $param["SysUser"];
			$this->password_api = $param["SysPwd"];
			$this->user = $param["LoginName"];
			$this->password = $param["Password"];		
			return true;
		}
		else
		{
			echo "ERROR => ".$data['Message']." \n";
			echo "ERROR => ".$data['ExptType']." \n";
			return false;
		}
	}

	/**
	 * getListBackupJobs
	 * @param string $url url du serveur
	 * @param string $param array contenant (login,pwd,...)	 
	 * @retval si authentification OK return true	 
	 */
	public function getListBackupJobs($url,$param)
	{	
		$data = self::get($url."/obs/api/json/ListBackupJobs.do",$param);
		$data = self::verifyData($data);
		return $data;
	}

	/**
	 * getListBackupJobStatus
	 * @param string $url url du serveur
	 * @param string $param array contenant (login,pwd,...)	 
	 * @retval si authentification OK return true	 
	 */
	public function getListBackupJobStatus($url,$param)
	{	
		$data = self::get($url."/obs/api/json/ListBackupJobStatus.do",$param);
		$data = self::verifyData($data);
		return $data;
	}

	/**
	 * getListBackupJobMode
	 * @param string $url url du serveur
	 * @param string $param array contenant (login,pwd,...)	 
	 * @retval si authentification OK return true	 
	 */
	public function getListBackupJobMode($url,$param)
	{	
		$data = self::get($url."/obs/api/json/ListBackupJobMode.do",$param);
		$data = self::verifyData($data);
		return $data;
	}

	/**
	 * getListBackupSets
	 * @param string $url url du serveur
	 * @param string $param array contenant (login,pwd,...)	 
	 * @retval si authentification OK return true	 
	 */
	public function getListBackupSets($url,$param)
	{	
		$data = self::get($url."/obs/api/json/ListBackupSets.do",$param);
		$data = self::verifyData($data);
		return $data;
	}

	/**
	 * getGetbackupSets
	 * @param string $url url du serveur
	 * @param string $param array contenant (login,pwd,...)	 
	 * @retval si authentification OK return true	 
	 */
	public function getGetbackupSets($url,$param)
	{	
		$data = self::get($url."/obs/api/json/GetBackupSet.do",$param);
		$data = self::verifyData($data);
		return $data;
	}	

	/**
	 * getBackupJobProgress
	 * @param string $url url du serveur
	 * @param string $param array contenant (login,pwd,...)	 
	 * @retval si authentification OK return true	 
	 */
	public function getBackupJobProgress($url,$param)
	{	
		$data = self::get($url."/obs/api/json/GetBackupJobProgress.do",$param);
		$data = self::verifyData($data);
		return $data;
	}

	/**
	 * GetUserStatus
	 * @param string $url url du serveur
	 * @param string $param array contenant (login,pwd,...)	 
	 * @retval si authentification OK return true	 
	 */
	public function getUserStatus($url,$param)
	{	
		$data = self::get($url."/cbs/api/json/GetUserStatus.do",$param);
		$data = self::verifyData($data);
		return $data;
	}
}

?>