#!/usr/bin/php
<?php
$options = getopt('A:M:O:');

$EXIT_SUCCESS=0;
$EXIT_NOT_IMPLEMENTED=-1;
$EXIT_ERROR=1;

if( !isset($options['M'])){
	echo "Parameter -M is missing\n"; exit($EXIT_ERROR);
}

$METHOD=$options['M'];
$action = strtolower($METHOD);

if(isset($options['O'])){
	if(is_array($options['O'])){
		$OPTIONS=$options['O'];
	}
	else{
		$OPTIONS=array($options['O']);
	}
}
else{
	$OPTIONS=array();
}

if( isset($options['A']) && is_readable($options['A'])){
        $options = json_decode(file_get_contents($options['A']),true);
}
else $options === FALSE;

foreach($OPTIONS as $opt){
	$exp=explode('=',$opt);
	$exp[0] = trim($exp[0]);
	$exp[1] = trim($exp[1]);

	if(count($exp) != 2 ) {
		echo "Incorrect Option -O format ($opt) must have format -O <name>=<value> -O <name>=<value> ... \n"; exit($EXIT_ERROR);
	}
	if( isset($options[$exp[0]]) ){
		echo "Option -O $opt overwrite another options parameter or auth file option\n"; exit($EXIT_ERROR);
	}

	$options[$exp[0]]=$exp[1];
}

if( $options === FALSE ){
        echo "Unable to decode auth JSON file $AUTH_FILE\n"; exit($EXIT_ERROR);
}
if( empty($options['host']) ){
        echo "Missing required 'host' parameter\n"; exit($EXIT_ERROR);
}
if( empty($options['dbname']) ){
        echo "Missing required 'dbname' parameter\n"; exit($EXIT_ERROR);
}
if( empty($options['dbuser']) ){
        echo "Missing required 'dbuser' parameter\n"; exit($EXIT_ERROR);
}
if( empty($options['dbpass']) ){
        echo "Missing required 'dbpass' parameter\n"; exit($EXIT_ERROR);
}

switch($action){
	case 'getjobsids' :
		$ret = getJobs($result,$options);
		break;
	case 'getjobsvalues' :
		$ret = getJobsValues($result,$options,true);
		break;
	default :
		echo "Method $action not implemented\n"; exit($EXIT_NOT_IMPLEMENTED);
}

echo json_encode($result)."\n";
exit($ret);

function getJobsByStatus($options,$where_status){
	$request = 'SELECT sv.id FROM [Backup.Model.BackupTaskSessions] sv';
	$request.= " WHERE $where_status";
	
	if(isset($options['period'])){
		$date = parseDate($options['period']);
		$request.= " AND sv.creation_time>='$date'";
	}
	if( isset($options['except'] )){
		$except = explode(',',$options['except']);
		$arr = array();
		foreach($except as $ex){
			$ex=trim($ex,"'");
			$arr[]="'$ex'";
		}
		$except = implode(',',$arr);
		$request.= " AND sv.id NOT IN($except)";
	}

	$response = execute_request($options,$request);
	$ids = array();
	foreach($response as $row){
		$ids[]=$row['id'];
	}
	return $ids;	
}

function getJobs(&$result,$options){
	if( isset($options['status']) && strtolower($options['status']) != 'all'){
		$status = explode(',',$options['status']);
		$jobs = array();
		foreach($status as $st){
			if( $st == 0 ){ //OK
				$where = 'sv.status NOT IN(2,3)';
				$jobs = array_merge($jobs,getJobsByStatus($options,$where));
			}
			else if($st == 1 ){ // WARN
				$where = 'sv.status = 3';
				$jobs = array_merge($jobs,getJobsByStatus($options,$where));
			}
			else if($st == 2 ){
				$where = 'sv.status = 2'; //CRITIC
				$jobs = array_merge($jobs,getJobsByStatus($options,$where));
			}
		}
	}
	else {
		$jobs = getJobsByStatus($options,'sv.status >= 0'); //Tous les status sont plus grand que 0
	}
	
	$result['ids']=$jobs;
	return $GLOBALS['EXIT_SUCCESS'];
}

function getJobsValues(&$result,$options){
	if( isset($options['ids']) ){
		$jobsids = explode(',',$options['ids']);
		$arr = array();
		foreach($jobsids as $ex){
			$ex=trim($ex,"'");
			$arr[]="'$ex'";
		}
		$jobsids = implode(',',$arr);	
	}
	else{
		echo "Parameter ids is missing\n";
		exit($GLOBALS['EXIT_ERROR']);
	}
	
	$request = "SELECT ";
	$request.= "sv.id AS id,";
	$request.= "js.job_name AS name,";
	$request.= "sv.object_name AS source,";
	$request.= "sv.stored_size AS size,";
	$request.= "(CASE js.job_type WHEN 0 THEN 'BACKUP' WHEN 1 THEN 'REPLICATION' WHEN 51 THEN 'COPY JOB'ELSE 'UNKNOWN' END) AS type,";
	$request.= "(CASE sv.status WHEN 0 THEN 'Success' WHEN 2 THEN 'Error' WHEN 3 THEN 'Warning' ELSE 'Success' END) AS status,";
	$request.= "sv.reason AS reason,";
	$request.= "sv.creation_time AS start_time,";
	$request.= "sv.end_time AS end_time";
	 
	$request.= " FROM [Backup.Model.BackupTaskSessions] sv INNER JOIN [Backup.Model.JobSessions] js ON sv.session_id = js.id";
	$request.= " WHERE sv.id IN($jobsids)";
	
	$response = execute_request($options,$request);
	$result = array();
	foreach($response as $row){
		$jobs['intern_id']	= $row['id'];
		$jobs['name'] 		= $row['name'];
		$jobs['source'] 	= $row['source'];
		$jobs['size'] 		= $row['size'];
		$jobs['type'] 		= $row['type'];
		
		switch(strtolower(trim($row['status']))){
			case 'success' :
				$jobs['status_esia'] = 0;
				break;
			case 'warning' :
				$jobs['status_esia'] = 1;
				break;
			case 'error' :
				$jobs['status_esia'] = 2;
				break;
		}
		
		if(empty($row['reason'])){
			$jobs['status_backup'] = $row['status'];
		}
		else{
			$jobs['status_backup'] = $row['status'] . " : " . $row['reason'];
		}

		$jobs['start_time'] = date_format(new DateTime($row['start_time']),'U');
		$jobs['end_time'] = date_format(new DateTime($row['end_time']),'U');
		if( $jobs['end_time'] < 0 ) $jobs['end_time'] = NULL; 
		
		$result[] = $jobs;
	}
	return $GLOBALS['EXIT_SUCCESS'];
}

function execute_request($options,$request){
	//Mode mssql instance
	if(isset($options['dbinstance'])){
		$host = $options['host'];
		$dbinstance = $options['dbinstance'];
		$dbuser = $options['dbuser'];
		$dbpass = $options['dbpass'];
		$dbname = $options['dbname'];

		try{
		        $db=new PDO("dblib:host=${host}\\${dbinstance};database=${dbname}", $dbuser, $dbpass);
		        $db->setAttribute(PDO::ATTR_ERRMODE,PDO::ERRMODE_WARNING);
		        $db->setAttribute(\PDO::DBLIB_ATTR_STRINGIFY_UNIQUEIDENTIFIER, true);
		
		        $sth=$db->prepare($request);
		        $sth->execute();
		
		        $ret = array();
		        while($row=$sth->fetch(PDO::FETCH_ASSOC))
		        {
                                $ret[] = $row;

		        }
		        return $ret;
	        }
	        catch(Exception $ex){
		        echo "DB access or SQL ERROR : ".$ex->getMessage()."\n"; exit($GLOBALS['EXIT_ERROR']);
	        }
	}
	
	//Mode with port 1433
	try{
		$host = $options['host'];
		$dbname = $options['dbname'];
		$dbuser = $options['dbuser'];
		$dbpass = $options['dbpass'];
		$dbport = $options['port'];
		
		$db = new PDO("dblib:version=7.0;charset=UTF-8;host=$host:$dbport;dbname=$dbname",$dbuser,$dbpass);
		$db->setAttribute(PDO::ATTR_ERRMODE,PDO::ERRMODE_WARNING);
		$db->setAttribute(\PDO::DBLIB_ATTR_STRINGIFY_UNIQUEIDENTIFIER, true);
		
		$sth=$db->prepare($request);
		$sth->execute();
		
		$ret = array();
		while($row=$sth->fetch(PDO::FETCH_ASSOC))
		{
			$ret[]=$row;
		}
		return $ret;
	}
	catch(Exception $ex){
		echo "DB access or SQL ERROR : ".$ex->getMessage()."\n"; exit($GLOBALS['EXIT_ERROR']);
	}
}

function parseDate($start){
	$start = strtolower($start);
	if(preg_match('/^[-]?(\d+)([hdwmy]?)$/',$start,$matches)){
		if(!empty($matches[2])){
			$matches[2]=strtoupper($matches[2]);
			switch($matches[2]){
				case 'H' :
					$since = time() - $matches[1] * 3600;
					break;
				case 'D' :
					$since = time() - $matches[1] * 86400;
					break;
				case 'W' :
					$since = time() - $matches[1] * 604800;
					break;
				case 'M' :
					$date = new DateTime();
					$date->sub(new DateInterval('P'.$matches[1].$matches[2]));
					$since=$date->format('U');
					break;
				case 'Y' :
					$date = new DateTime();
					$date->sub(new DateInterval('P'.$matches[1].$matches[2]));
					$since=$date->format('U');
					break;
			}
		}
		else $since = $matches[1];
	}
	else throw new Exception();

	return date('Y-m-d H:i:s',$since);
}
?>