<?php
$path = $_SERVER['DOCUMENT_ROOT'];
include_once MVCMODEL;

class Model_Backup extends Model_esia{
	private static $status_list = array('ok','warning','critical','unknown');

	public static function removeAllLinks($dbconnection,$drv_config_id){
		$sql = "DELETE FROM backup_devicelink WHERE bdev_config_id=:drv_id";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,array('drv_id'=>$drv_config_id));
	}

	public static function getDevicesList($dbconnection,$drv_config_id,$type){
		$sql = 'SELECT bdev_id,bdev_name,bdev_data,bdev_device_id FROM backup_devicelink WHERE bdev_config_id = :config AND bdev_type = :type';
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,array(':config'=>$drv_config_id,':type'=>$type));
		$ret = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$ret[$row->bdev_name] = array(
					'id' => $row->bdev_id,
					'name' => $row->bdev_name,
					'data' => $row->bdev_data,
					'link' => $row->bdev_device_id,
			);
		}
		return $ret;
	}

	private function genericTableDeviceLinkSqlAction($dbconnection,$drv_config_id,$type,$devices,$sql){
		$ret['success']=array();
		$ret['error']=array();

		$query = $dbconnection->prepare($sql);

		foreach($devices as $name => $data){
			if( is_array($data) ) $data = json_encode($data);
			try{
				$query = @Model_esia::executeStatement($query,array(':config'=>$drv_config_id,':device'=>$name,':type'=>$type,':data'=>$data));
				$ret['success'][$name] = $data;
			}
			catch(Exception $ex){
				$ret['error'][$name] = $data;
				continue;
			}
		}
		return $ret;
	}

	public static function addDevices($dbconnection,$drv_config_id,$type,$devices){
		$sql = 'INSERT INTO backup_devicelink (bdev_config_id,bdev_name,bdev_type,bdev_data) VALUES(:config,:device,:type,:data)';
		return self::genericTableDeviceLinkSqlAction($dbconnection,$drv_config_id,$type,$devices,$sql);
	}

	public static function updateDevices($dbconnection,$drv_config_id,$type,$devices){
		$sql = 'UPDATE backup_devicelink SET bdev_data = :data
				WHERE bdev_config_id = :config AND bdev_type = :type AND bdev_name = :name';
		return self::genericTableDeviceLinkSqlAction($dbconnection,$drv_config_id,$type,$devices,$sql);
	}

	public static function addJobList($dbconnection,$arr,$device_id){
	    //$keys=':serial,:name,:status_esia,:start_time,:end_time,:source,:storage,:size,:type,:status_backup';
	    $ret['total']=0;
	    $ret['added']=0;
	    $ret['updated']=0;
	    $ret['error']=0;

	    $keys = array(
	        'job_soft_id' 	=> ':intern_id',
	        'job_server_id'	=> ':srv_id',
	        'job_name' 		=> ':name',
	        'job_status' 	=> ':status_esia',
	        'job_start_time'=> ':start_time',
	        'job_end_time'	=> ':end_time',
	        'job_source'	=> ':source',
	        'job_size'		=> ':size',
	        'job_type'		=> ':type',
	        'job_messages'	=> ':status_backup'
	    );
	    $fields_name = implode(',',array_keys($keys));

	    $tmp = $keys;
	    $tmp['job_start_time']='to_timestamp(:start_time)';
	    $tmp['job_end_time']='to_timestamp(:end_time)';
	    $values = implode(',',array_values($tmp));

	    $sql="INSERT INTO backup_job ($fields_name) VALUES ($values) ON CONFLICT ON CONSTRAINT job_srv_soft_id_unique DO NOTHING";
	    $jobquery = $dbconnection->prepare($sql);

	    foreach($arr as $job){//Traitement des jobs
	        $ret['total']++;

	        try{
	            $job['srv_id'] = $device_id;

	            $params = array();
	            foreach($keys as $key){
	                $key = explode(':',$key)[1];
	                if( isset($job[$key])) $params[":$key"] = $job[$key];
	                else $params[":$key"] = NULL;
	            }

	            $query = @Model_esia::executeStatement($jobquery,$params);
	            $ret['added']++;
	        }
	        catch(Exception $ex){
	            if( self::isIntoProgressJob($dbconnection,$job['intern_id'],$device_id,$job['start_time']) ){
	                try{
	                    $ret['updated']++;
	                    self::updateIntoProgressJob($dbconnection,$job);
	                }
	                catch(Exception $ex){
	                    $ret['error']++;
	                }
	            }
	            else{
	                $ret['error']++;
	            }
	            continue;
	        }
	    }
	    return $ret;
	}

	public static function getInProgressJobs($dbconnection,$host_id,$days){
        $sql = "SELECT job_soft_id FROM backup_job WHERE job_server_id=:job_server_id AND job_end_time IS null AND job_start_time > (current_date - '$days day'::interval) ORDER BY job_id ASC";
        $query = $dbconnection->prepare($sql);
        $query = Model_esia::executeStatement($query,array(':job_server_id'=>$host_id));
        $ret = array();
        while($row=$query->fetch(PDO::FETCH_OBJ)){
                $ret[]=$row->job_soft_id;
        }
        return $ret;
}

	private static $isIntoProgressJobSql=NULL;
	public static function isIntoProgressJob($dbconnection,$job_soft_id,$job_server_id,$job_start_time){
	    try{
	        if( self::$isIntoProgressJobSql == NULL ) {
	            $sql = 'SELECT count(*) AS NB FROM backup_job WHERE job_soft_id=:job_soft_id AND job_server_id=:job_server_id AND job_start_time=to_timestamp(:job_start_time) AND job_end_time IS NULL';
	            self::$isIntoProgressJobSql = $dbconnection->prepare($sql);
	        }
	        $result = @Model_esia::executeStatement(self::$isIntoProgressJobSql,array(':job_soft_id'=>$job_soft_id,':job_server_id'=>$job_server_id,':job_start_time'=>$job_start_time));
	        $row = $result->fetch(PDO::FETCH_ASSOC);
	        if( isset($row['nb'])&&$row['nb'] == 1){
	            return TRUE;
	        }
	    }
	    catch(Exception $ex){
	    }
	    return FALSE;
	}

	private static $updateIntoProgressJobSql=NULL;
	public static function updateIntoProgressJob($dbconnection,$newValues){
	    if( self::$updateIntoProgressJobSql == NULL ){
	        $sql = 'UPDATE backup_job';
	        $sql.= ' SET job_name=:job_name,job_status=:job_status,job_end_time=to_timestamp(:job_end_time),job_size=:job_size,job_type=:job_type,job_messages=:job_messages';
	        $sql.= ' WHERE job_soft_id=:job_soft_id AND job_server_id=:job_server_id AND job_start_time=to_timestamp(:job_start_time) AND job_end_time IS NULL';
	        self::$updateIntoProgressJobSql = $dbconnection->prepare($sql);
	    }
	    $params = array(
	        //SET
	        ':job_name' => $newValues['name'],
	        ':job_status' => $newValues['status_esia'],
	        ':job_end_time' => $newValues['end_time'],
	        ':job_size' => $newValues['size'],
	        ':job_type' => $newValues['type'],
	        ':job_messages' => $newValues['status_backup'],
	        //WHERE
	        ':job_soft_id' => $newValues['intern_id'],
	        ':job_server_id' => $newValues['srv_id'],
	        ':job_start_time' => $newValues['start_time']
	    );

	    Model_esia::executeStatement(self::$updateIntoProgressJobSql,$params);
	}

	public function lastJobStatus($dbconnection,$server_id,$since){
		$sql = 'SELECT job_status,COUNT(*) AS nb FROM backup_job ';
		$sql.= 'WHERE job_server_id=:srv_id AND (job_end_time > to_timestamp(:since) OR job_start_time > to_timestamp(:since))';
		$sql.= 'GROUP BY job_status';

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,array(':since'=>$since,':srv_id'=>$server_id));

		$ret = array(0,0,0,0,0);
		while($row=$query->fetch(PDO::FETCH_OBJ)){
			$ret[$row->job_status] = $row->nb;
		}
		return $ret;
	}

	public function getBackupPeriod($dbconnection,$server_id){
		$sql = "SELECT date_trunc('day',MIN(job_start_time)) as ".'"first"'.", date_trunc('day',MAX(job_start_time)) as ".'"last"';
		$sql.= " FROM backup_job WHERE job_server_id=:srv_id";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,array(':srv_id'=>$server_id));

		$row=$query->fetch(PDO::FETCH_OBJ);

		$first =  new DateTime($row->first);
		$last = new DateTime($row->last);

		return array('first'=>$first->format('U'),'last'=>$first->format('U'));
	}

	public function getAllBackupsName($dbconnection,$server_id,$type){
		if( strtolower($type) == 'source' ){
			$type= 'job_source';
		}
		else if (strtolower($type) == 'job'){
			$type = 'job_name';
		}
		else throw new Exception('You must specify correct $type "source" or "job"');

		$sql = "SELECT DISTINCT $type FROM backup_job WHERE job_server_id=:srv_id ORDER BY $type ASC";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,array(':srv_id'=>$server_id));
		return $query->fetchAll(PDO::FETCH_COLUMN,0);
	}

	function getAllBackupByTypeSince($dbconnection,$server_id,$type,$since,$until=null){
		if( strtolower($type) == 'source' ){
			$type= 'job_source';
		}
		else if (strtolower($type) == 'job'){
			$type = 'job_name';
		}
		else throw new Exception('You must specify correct $type "source" or "job"');

		$params = array(':srv_id'=>$server_id,':since'=>$since);

		$sql = "SELECT $type AS job_name,job_start_time as date,job_status as status";
		$sql.= " FROM backup_job";
		$sql.= " WHERE job_server_id=:srv_id AND job_start_time > to_timestamp(:since)";
		if( $until != null ){
			$params[':until'] = $until;
			$sql.= " AND job_start_time < to_timestamp(:until)";
		}

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);

		$ret=array();
		while($row=$query->fetch(PDO::FETCH_OBJ)){
			$ret[] = array(
					'name' => $row->job_name,
					'date' => $row->date,
					'status' => $row->status,
			);
		}
		return $ret;

	}

	public function getAllBackupsByTypeAndPeriod($dbconnection,$server_id,$type,$period='day'){
		if( strtolower($type) == 'source' ){
			$type= 'job_source';
		}
		else if (strtolower($type) == 'job'){
			$type = 'job_name';
		}
		else throw new Exception('You must specify correct $type "source" or "job"');

		if( $period != 'hour' ){
			$period='day';
		}

		$sql = "SELECT $type AS job_name,date_trunc('$period', job_start_time) AS date,job_status";
		$sql.= " FROM backup_job WHERE job_server_id=:srv_id";
		$sql.= " GROUP BY $type,date,job_status";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,array(':srv_id'=>$server_id));

		$ret=array();
		while($row=$query->fetch(PDO::FETCH_OBJ)){
			$ret[] = array(
					'name' => $row->job_name,
					'date' => $row->date,
					'status' => $row->job_status,
			);
		}
		return $ret;
	}

	// Mode :
	// -1 => ERROR
	// 1  => OK
	// 0 => ALL
	public function getBackups($dbconnection,$server_id,$limit=NULL,$mode=0){
		$params = array(':srv_id'=>$server_id);
		$sql = 'SELECT * FROM backup_job WHERE job_server_id=:srv_id ';
		if($mode < 0){ //errors only
			$sql.= ' AND job_status > 0';
		}
		else if($mode > 0){ // Success only
			$sql.= ' AND job_status = 0';
		}
		$sql.= ' ORDER BY GREATEST(job_end_time,job_start_time)';
		if(is_int($limit)){
			$sql.= ' DESC limit :limit';
			$params[':limit']=$limit;
		}

		$query = $dbconnection->prepare($sql);
		return self::executeDefaultBackupQuery($query,$params);
	}

	public static function getJobMessages($dbconnection,$jobid){
		$sql = 'SELECT job_messages FROM backup_job WHERE job_id=?';
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,array($jobid));
		$row=$query->fetch(PDO::FETCH_OBJ);
		return json_decode($row->job_messages,true);
	}

	public static function removeAllJobs($dbconnection,$srv_id){
		$sql = "DELETE FROM backup_job WHERE job_server_id=:srv_id";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,array('srv_id'=>$srv_id));
	}

	public static function removeOldJobs($dbconnection,$srv_id,$days){
		$sql = "DELETE FROM backup_job WHERE job_end_time < to_timestamp(:ts) AND job_server_id=:srv_id";

		$interval = $days * 86400; //3600s * 24
		$since = time() - $interval;

		$sta = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($sta,array('srv_id'=>$srv_id,':ts'=>$since));
		return $sta->rowCount();
	}

	public static function getStoragesList($dbconnection,$srv_id){
		$sql = 'SELECT DISTINCT job_storage_name from backup_job WHERE job_server_id=?';
		$query = $dbconnection->prepare($sql);

		$query = Model_esia::executeStatement($query,array($srv_id));

		$ret = array();
		while($row=$query->fetch(PDO::FETCH_OBJ)){
			$ret[] = $row->job_storage_name;
		}
		return $ret;
	}

	private static function executeDefaultBackupQuery($query,$params){
		$query = Model_esia::executeStatement($query,$params);

		$ret = array();
		while($row=$query->fetch(PDO::FETCH_OBJ)){
			$duration = null;
			$job_end = null;
			$dt = new DateTime($row->job_start_time);
			$job_start = $dt->format('U');

			if($row->job_end_time != null) {
				$dt = new DateTime($row->job_end_time);
				$job_end = $dt->format('U');
				$duration = $job_end - $job_start;
			}

			$ret[] = array(
					'id'			=> $row->job_id,
					'intern_id' 	=> $row->job_soft_id,
					'name' 			=> $row->job_name,
					'esia_status'	=> $row->job_status,
					'intern_status' => self::$status_list[$row->job_status],
					'message'		=> $row->job_messages,
					'start_time'	=> $job_start,
					'end_time'		=> $job_end,
					'duration'		=> $duration,
					'server_id'		=> $row->job_server_id,
					'source'		=>$row->job_source,
					'storage'		=>$row->job_storage,
					'size'			=> $row->job_size,
					'type'			=> $row->job_type,
			);
		}
		return $ret;
	}

	private static function lastjobbynodes($dbconnection,$group,$srv_id){
		$sql = "SELECT * FROM backup_job WHERE job_start_time IN
				(SELECT MAX(job_start_time) as job_id FROM backup_job  WHERE job_server_id=? AND $group IS NOT NULL GROUP BY $group)";

		$query = $dbconnection->prepare($sql);
		return self::executeDefaultBackupQuery($query,array($srv_id));
	}

	public static function lastjobbystorages($dbconnection,$srv_id){
			return self::lastjobbynodes($dbconnection,'job_storage',$srv_id);
	}

	public static function lastjobbysources($dbconnection,$srv_id){
		return self::lastjobbynodes($dbconnection,'job_source',$srv_id);
	}

	public static function lastjobbynames($dbconnection,$srv_id){
		return self::lastjobbynodes($dbconnection,'job_name',$srv_id);
	}

	private static function genericCalculs($dbconnection,$group,$mode,$item,$since,$until,$server,$filters=array()){
		$where = '';
		$params = array(':server'=>$server,':since'=>$since,':until'=>$until);
		foreach($filters as $key=>$filter){
			if(is_int($filter) ) {
				$where.=' AND '.$key.' = :p_'.$key;
				$params[":p_$key"] = $filter;
			}
			else {
				$where.=' AND '.$key.' like :p_'.$key;
				$params[":p_$key"] = "%$filter%";
			}
		}

		$sql = "SELECT $group,$mode($item) AS value FROM backup_job";
		$sql.= ' WHERE job_server_id=:server';
		$sql.= ' AND job_end_time BETWEEN to_timestamp(:since) AND to_timestamp(:until)';
		$sql.= $where;
		$sql.= " GROUP BY $group";
		$sql.= " ORDER BY $mode($item) ASC";
		/*
		echo "<pre>";
		echo "$sql\n";
		print_r($params);
		echo "</pre>";
		return;
	*/
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);

		$ret = array();
		while($row=$query->fetch(PDO::FETCH_ASSOC))
		{
			$ret[$row[$group]] = $row['value'];
		}
		return $ret;
	}

	private static function genericDurations($dbconnection,$group,$since,$until,$server,$filters=array()){
		$where = '';
		$params = array(':server'=>$server,':since'=>$since,':until'=>$until);
		foreach($filters as $key=>$filter){
			$where.=' AND '.$key.' like :p_'.$key;
			$params[":p_$key"] = "%$filter%";
		}

		$sql = "SELECT $group,";
		$sql.= " MIN(DATE_PART('second', job_end_time - job_start_time)),";
		$sql.= " AVG(DATE_PART('second', job_end_time - job_start_time)),";
		$sql.= " MAX(DATE_PART('second', job_end_time - job_start_time)),COUNT(job_id)";
		$sql.= " FROM backup_job";
		$sql.= ' WHERE job_server_id=:server';
		$sql.= ' AND job_end_time BETWEEN to_timestamp(:since) AND to_timestamp(:until)';
		$sql.= $where;
		$sql.= " GROUP BY $group";
		$sql.= " ORDER BY $group";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);

		$ret = array();
		while($row=$query->fetch(PDO::FETCH_ASSOC))
		{
			$key = $row[$group];
			unset($row[$group]);
			$ret[$key] = $row;
		}
		return $ret;
	}

	private static function genericStats($dbconnection,$group,$item,$since,$until,$server,$filters=array()){
		$where = '';
		$params = array(':server'=>$server,':since'=>$since,':until'=>$until);
		foreach($filters as $key=>$filter){
			$where.=' AND '.$key.' like :p_'.$key;
			$params[":p_$key"] = "%$filter%";
		}

		$sql = "SELECT $group,MIN($item),AVG($item),MAX($item),SUM($item),COUNT($item) FROM backup_job";
		$sql.= ' WHERE job_server_id=:server';
		$sql.= ' AND job_end_time BETWEEN to_timestamp(:since) AND to_timestamp(:until)';
		$sql.= $where;
		$sql.= " GROUP BY $group";
		$sql.= " ORDER BY $group";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);

		$ret = array();
		while($row=$query->fetch(PDO::FETCH_ASSOC))
		{
			$key = $row[$group];
			unset($row[$group]);
			$ret[$key] = $row;
		}
		return $ret;
	}

	public static function getNodesStats($dbconnection,$type,$server_id,$since=null,$until=null){
		if($since == null) $since = 0;
		if($until == null ) $until = time();

		$type = trim($type);
		if( $type == 'job_storage' || $type == 'storage' ) 		$type = 'job_storage';
		else if( $type == 'job_source' || $type == 'source' ) 	$type = 'job_source';
		else if( $type == 'job_name' || $type == 'job' ) 		$type = 'job_name';
		else throw new Exception();

		$sizes = self::genericStats($dbconnection,$type,'job_size',$since,$until,$server_id);
		$durations = self::genericDurations($dbconnection,$type,$since,$until,$server_id);
		$first_dates = self::genericCalculs($dbconnection,$type,'MIN','job_end_time',$since,$until,$server_id);
		$last_dates = self::genericCalculs($dbconnection,$type,'MAX','job_end_time',$since,$until,$server_id);
		$last_ok = self::genericCalculs($dbconnection,$type,'MAX','job_end_time',$since,$until,$server_id,array('job_status' => 0));
		$last_warn = self::genericCalculs($dbconnection,$type,'MAX','job_end_time',$since,$until,$server_id,array('job_status' => 1));

		$data = array();
		foreach($sizes as $name => $size){
			$node['name'] = $name;
			$node['first_date'] = $first_dates[$name];
			$node['last_date'] = $last_dates[$name];
			if(isset($last_ok[$name])) $node['last_ok'] = $last_ok[$name];
			else $node['last_ok'] = null;
			if(isset($last_warn[$name])) $node['last_warn'] = $last_warn[$name];
			else $node['last_warn'] = null;
			foreach($size as $key => $value){
				$node["size_$key"] = $value;
			}
			foreach($durations[$name] as $key => $value){
				$node["duration_$key"] = $value;
			}

			$states = self::genericCalculs($dbconnection,'job_status','COUNT','job_id',$since,$until,$server_id,array($type=>$name));
			for($i=0;$i<=2;$i++) if(empty($states[$i])) $states[$i] = 0;

			$node['ok'] = $states[0];
			$node['warn'] = $states[1];
			$node['crit'] = $states[2];
			$node['unk'] = $states[3];
			$node['pend'] = $states[4];

			$data[]=$node;
		}
		return $data;
	}

	public static function getAllNodesStats($dbconnection,$server_id,$since=null,$until=null){
		$storages = self::getNodesStats($dbconnection,'job_storage',$server_id,$since,$until);
		$clients  = self::getNodesStats($dbconnection,'job_source',$server_id,$since,$until);

		$data = array();
		foreach($storages as $storage){
			$storage['type'] = 'storage';
			$data[] = $storage;
		}
		foreach($clients as $source){
			$source['type'] = 'source';
			$data[] = $source;
		}
		return $data;
	}

	public static function getJobSourcesList($dbconnection,$device_id){
		$sql = "SELECT job_name,job_source FROM backup_job WHERE job_server_id = ?";
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,array($device_id));
		return $query->fetchAll(PDO::FETCH_ASSOC);
	}
}
