<?php
$path = $_SERVER['DOCUMENT_ROOT'];
include_once MVCMODEL;
include_once $_SERVER['DOCUMENT_ROOT'].'Model_adm.php';
include_once $_SERVER['DOCUMENT_ROOT'].'../keystore/scripts/keystore.lib.php';

class Model_BackupSettings extends Model_esia{
	private static function getUserKey($dbconnection){
		$conf_key =  getConfKey();
		$e_uk=getEncryptedUserKey($dbconnection);
		return decryptUserKey($conf_key,$e_uk);
	}
	
	private static function decryptFile($dbconnection,$path,$settings_id=null){
		//return loadData(self::getUserKey($dbconnection),$path);
		return loadSettings($dbconnection,$settings_id);
	}
	
	private static function encryptFile($dbconnection,$path,$data,$settings_id=null){
		//saveData(self::getUserKey($dbconnection),$path,$data);
		insertSettings($dbconnection,$settings_id,$path,$data,true);
	}
	
	public static function getDrivers($dbconnection,$name=null,$type=null){
		$sql = 'SELECT * FROM backup_driver';
		$arr = array();
		$where = null;
		
		if( $name != null ){
			$arr[':name'] = $name;
			$where = 'drv_name = :name';
		}
		if( $type != null ){
			$arr[':type'] = $type;
			if($where == null) $where = 'drv_type = :type';
			else $where.=' AND drv_type = :type';
		}
		
		if ( $where != null) $sql.= " WHERE $where";
		
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$arr);
		
		$ret = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$ret[] = array(
					'id' => $row->drv_id,
					'name' => $row->drv_name,
					'type' => $row->drv_type,
					'path' => $row->drv_path,
					'admin'=> $row->drv_admin_page
			);
		}
		return $ret;
	}
	
	public static function getDriver($dbconnection,$name,$type){
		$ret = self::getDrivers($dbconnection, $name, $type);
		if( is_array($ret)) $ret = array_pop($ret);
		return $ret;
	}
	
	public static function getDriverById($dbconnection,$id){
		$sql = 'SELECT * FROM backup_driver WHERE drv_id = ?';
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,array($id));
		$row = $query->fetch(PDO::FETCH_OBJ);
		if( !$row ) return null;
		return array(
				'id' => $row->drv_id,
				'name' => $row->drv_name,
				'type' => $row->drv_type,
				'path' => $row->drv_path,
				'admin'=> $row->drv_admin_page
		);
	}
	
	public static function getDeviceDriverSettings($dbconnection,$device_id=null){
		//Récupération des driveurs
		$drivers = array();
		foreach(self::getDrivers($dbconnection) as $drv){
			$drivers[$drv['id']] = $drv;
		}
		
		$data = self::getDeviceAdvanceConfig($dbconnection,$device_id);
		$ret = array();
		foreach($data as $row){
			if($row['drv_id'] != NULL ){
				$driver = $drivers[$row['drv_id']];
				$ret[] = array(
						'drv'  => $row['drv_id'],
						'name' => $driver['name'],
						'host_id' => $row['host_id'],
						'host' => $row['host'],
						'type' => $driver['type'],
						'path' => $driver['path'],
						'authfile' => $row['authfile'],
						'params' => self::getConfig($dbconnection,$row['drv_id'],$row['host_id']),
						'retention' => $row['retention'],
				);
			}
		}
		return $ret;
	}
	
	public static function getDeviceAdvanceConfig($dbconnection,$device_id=null){
		$sql = 'SELECT adv_id,device_id,device_ip_address,adv_param,adv_authfile_crypt';
		$sql.= ' FROM esia_device_advance_config';
		$sql.= ' INNER JOIN esia_device ON device_id=adv_device_id';
		$sql.= " WHERE device_type like 'backup server'";
		
		$params = array();
		if( $device_id != null ) {
			$sql.= ' AND adv_device_id = :dev';
			$params[':dev']=$device_id;
		}
		
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,$params);
		
		$ret = array();
		while($row=$query->fetch(PDO::FETCH_OBJ)){
			if(empty($row->adv_param)){
				$bck_params['drv_id'] = NULL;
				$bck_params['retention'] = NULL;
				$bck_params['stats'] = NULL;
			}
			else{
				$bck_params = json_decode($row->adv_param,true);
			}
			
			$ret[] = array(
					'id'            => $row->adv_id,
					'authfile'      => $row->adv_authfile_crypt,
					'drv_id'        => $bck_params['drv_id'],
					'retention'     => $bck_params['retention'],
					'stats'         => $bck_params['stats'],
					'host_id'       => $row->device_id,
					'host'          => $row->device_ip_address,
			);
		}
		return $ret;
	}
	
	public static function updateAdvanceConfig($dbconnection,$adv_id,$driver_id,$device_id,$data,$retention,$stats){
		$driver = self::getDriverById($dbconnection,$driver_id);
		
		$adv_params['drv_id'] = $driver_id;
		$adv_params['retention'] = $retention;
		$adv_params['stats'] = $stats;
		$file_path = 'backup/'.$driver['name'].'_'.$driver['type']."_$device_id";
		
		$sql = 'UPDATE esia_device_advance_config';
		$sql.= ' SET adv_param = :params';
		$sql.= ',adv_authfile_crypt = :file_path ';
		$sql.= ' WHERE adv_id = :id';
		
		if( $data !== FALSE) self::encryptFile($dbconnection,$file_path,$data,$adv_id);
		
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,array(
				':id' => $adv_id,
				':params' => json_encode($adv_params),
				':file_path' => $file_path
		));
	}
	
	
	public static function removeAdvanceConfig($dbconnection,$adv_id,$driver_id,$device_id){
		$driver = self::getDriverById($dbconnection,$driver_id);
		$file_path = 'backup/'.$driver['name'].'_'.$driver['type']."_$device_id";
		
		$sql = 'UPDATE esia_device_advance_config';
		$sql.= ' SET adv_param = :params';
		$sql.= ',adv_authfile_crypt = :file_path ';
		$sql.= ' WHERE adv_id = :id';
		
		unlink(CIPHER_FILES_DIR."/$file_path.auth");
		unlink(CLEAR_FILES_DIR."/$file_path.dat");
		
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,array(
				':id' => $adv_id,
				':params' => NULL,
				':file_path' => NULL,
		));
	}
	
	public static function getConfig($dbconnection,$driver_id,$device_id){
		$settings = self::getDeviceAdvanceConfig($dbconnection,$device_id);
		foreach($settings as $s){
			if($s['drv_id'] == $driver_id ){
				return self::decryptFile($dbconnection,$s['authfile'],$s['id']);
			}
		}
		return NULL;
	}
	
	public static function addConfig($dbconnection,$driver_id,$device_id,$data,$retention){
		$settings = self::getDeviceAdvanceConfig($dbconnection,$device_id);
		foreach($settings as $s){
			if($s['drv_id'] == NULL ){
				self::updateAdvanceConfig($dbconnection,$s['id'],$driver_id,$device_id,$data,$retention,NULL);
				return;
			}
		}
	}
	
	public static function updateConfig($dbconnection,$driver_id,$device_id,$data){
		$settings = self::getDeviceAdvanceConfig($dbconnection,$device_id);
		foreach($settings as $s){
			if($s['drv_id'] == $driver_id ){
				self::updateAdvanceConfig($dbconnection,$s['id'],$driver_id,$device_id,$data,$s['retention'],$s['stats']);
				return;
			}
		}
	}
	
	public static function removeConfig($dbconnection,$driver_id,$device_id){
		$settings = self::getDeviceAdvanceConfig($dbconnection,$device_id);
		foreach($settings as $s){
			if($s['drv_id'] == $driver_id ){
				self::removeAdvanceConfig($dbconnection,$s['id'],$driver_id,$device_id);
				return true;
			}
		}
		return false;
	}
	
	public static function updateRetention($dbconnection,$driver_id,$device_id,$retention){
		$settings = self::getDeviceAdvanceConfig($dbconnection,$device_id);
		foreach($settings as $s){
			if($s['drv_id'] == $driver_id ){
				self::updateAdvanceConfig($dbconnection,$s['id'],$driver_id,$device_id,FALSE,$retention,$s['stats']);
				return TRUE;
			}
		}
		return FALSE;
	}
	
	public static function saveStats($dbconnection,$driver_id,$device_id,$data){
		$settings = self::getDeviceAdvanceConfig($dbconnection,$device_id);
		foreach($settings as $s){
			if($s['drv_id'] == $driver_id ){
				self::updateAdvanceConfig($dbconnection,$s['id'],$driver_id,$device_id,FALSE,$s['retention'],$data);
				return TRUE;
			}
		}
		return FALSE;
	}
}
