#!/usr/bin/php
<?php
/**
 * @auhtor Julien GEORGE <j.george@esia-sa.com
 * @section DESCRIPTION
 * Ce script est prévu pour être utilisé dans une tâche cron.
 * Soit sans paramètre, soit avec les paramètres suivants :
 *  - num tâche : Le numéro de la tâche exécuté
 *  - total tâches : Le nombre total de tâche a exécuter
 * Si total tâches = 4, le script doit absolument être lancé 4x. Exemple :
 * /usr/local/esia/public_html/plugin/backup/sync-backups.cron.php 0 4
 * /usr/local/esia/public_html/plugin/backup/sync-backups.cron.php 1 4
 * /usr/local/esia/public_html/plugin/backup/sync-backups.cron.php 2 4
 * /usr/local/esia/public_html/plugin/backup/sync-backups.cron.php 3 4
*/

$lock_path = "/tmp/esia_bck_cron.lock";
if( isset($argv[2]) && is_numeric($argv[1]) && is_numeric($argv[2]) && $argv[1] < $argv[2] ){
	$cron_num = $argv[1];
        $cron_total = $argv[2];

	$lock_path = "/tmp/esia_bck_cron$cron_num.lock";
}

$fp = fopen($lock_path, 'w+');
if(!flock($fp,LOCK_EX|LOCK_NB)){
	echo "Script is already used\n";
	exit(0);
}

define('PUBLIC_HTML','/usr/local/esia/public_html/');
define('DRIVER_DIR',__DIR__.'/drivers');
define('EXEC_TIMEOUT',1200);

$_SERVER['DOCUMENT_ROOT'] = PUBLIC_HTML;
$path = PUBLIC_HTML;

include_once $path."esia.conf.php";
include_once MVCMODEL;


$dbconnection = new PDO('pgsql:host='.DB_HOST.';port='.DB_PORT.';dbname='.DB_NAME, DB_USERNAME, DB_PWD);
$dbconnection->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_WARNING);

$arr = Model_esia::getWebPlugin($dbconnection,false,'Backup');
$include_path= $path.$arr[0]['apache_path'].'inc/';

include_once($include_path.'Model_Backup.php');
include_once($include_path.'Model_BackupSettings.php');

$settings=Model_BackupSettings::getDeviceDriverSettings($dbconnection);
$sorted = array();
foreach($settings as $value){
	$sorted[$value['host_id']] = $value;
}
krsort($sorted);
$settings = array_values($sorted);

if( isset($cron_num)){

	$tmp = array();
	foreach($settings as $key => $value){
		if( $value['host_id'] % $cron_total == $cron_num ){
			$tmp[]=$value;
		}
	}
	$settings = $tmp;
}

Class DriverException extends Exception{}
Class NotImplementedDriverException extends DriverException{}
Class ErrorDriverException extends DriverException{}
class UnknownDriverException extends DriverException{}

function run_driver($path_script,$auth_file,$method,$options=array()){
	$cmd = "$path_script -A $auth_file -M $method";
	foreach($options as $key => $value){
		$cmd.= " -O $key=$value";
	}
	$err=exec("timeout ".EXEC_TIMEOUT. " $cmd",$output,$ret);
	if( $ret == -1) 
		throw new NotImplementedDriverException("Method $method isn't implemented for driver $path_script : $err");
	if( $ret == 1 )
		throw new ErrorDriverException("Driver $path_script has returned fatal error exception : $err");
	if( $ret != 0 )
		throw new UnknownDriverException("Driver has returned unknown return code : $err");
	
	$json = implode("\n",$output);
	$data = json_decode($json,true);
	if($data == false )  throw new UnknownDriverException("Driver doesn't return json data");
	return $data;
}

function multi_run_driver($path_script,$auth_file,$method,$options=array()){
	//On relance x fois
	for($i=0;$i<2;$i++){
		try{
			return run_driver($path_script,$auth_file,$method,$options);
		}
		catch(Exception $ex){
			$wait_time = ( $i + 1 ) * 2; // 2 - 4 - 6 - 8 - 10
			sleep($wait_time);
		}
	}
	//Dernier essai
	return run_driver($path_script,$auth_file,$method,$options);
}

function getJobIds($gesasn,$path_script,$auth_file,$period,$status=null){
        if($gesasn!=NULL){
                $options['gesasn']=$gesasn;
        }
        $options['period']=$period;
        if( $status !== null) $options['status']=$status;
        return multi_run_driver($path_script,$auth_file,'getjobsids',$options)['ids'];
}

function getJobsFields($gesasn,$path_script,$auth_file,$ids){
        if($gesasn!=NULL){
                $options['gesasn']=$gesasn;
        }
        $options['ids'] = implode(',',$ids);
        return multi_run_driver($path_script,$auth_file,'getjobsvalues',$options);
}

foreach($settings as $config){
	try{
	$stats['time']['start'] = time();
	
	$deleted = 0;
	if( $config['retention'] > 0 ){
		$deleted=Model_Backup::removeOldJobs($dbconnection,$config['host_id'],$config['retention']);
	}
	$stats['backups']['deleted'] = $deleted;
	$stats['time']['cleaned'] = time();
	
	$lastjob=Model_Backup::getBackups($dbconnection,$config['host_id'],1);
	if(count($lastjob) == 1 ) {
		if( !empty($lastjob[0]['end_time']) ){
			$job_ts = $lastjob[0]['end_time'];
		}
		else{
			$job_ts = $lastjob[0]['start_time'];
		}
	}
	
	$now_ts = time();
	$hours = intval(( $now_ts - $job_ts ) / 3600);
	$hours+=96;
	
	if($hours > ( $config['retention'] * 24 ) && $config['retention'] != 0 ) $hours = $config['retention'] * 24;
	$stats['backups']['hours'] = $hours;
	
	$script = DRIVER_DIR.'/'.$config['name'].'/'.$config['path'];
	//$auth_file = DRIVER_DIR.'/'.$config['name']."/auth.txt";
	$auth_file = CLEAR_FILES_DIR.'/'.$config['authfile'].'.dat';
	
	echo "Taking $hours hours backups\n";
	
	$status_code = array(0,1,2,3);
	$stats['backups']['present'] = 0;
	$stats['backups']['updated'] = 0;
	$stats['backups']['added'] = 0;
	
	$gesasn=NULL;
	$gesa=Model_esia::getGesaLink($dbconnection,$config['host_id']);
	if( is_a($gesa,'Device') ){
		$gesasn=Model_esia::getGesaSN($dbconnection,$gesa->getId());
	}

	foreach($status_code as $st){
		$ids = getJobIds($gesasn,$script,$auth_file,$hours.'h',$st);
		if(!empty($ids)){
			echo "ids : " . count($ids)."\n";
			
			foreach(array_chunk($ids,50) as $chunk){
				echo "GET " . count($chunk) . " backup\n";
				$jobs = getJobsFields($gesasn,$script,$auth_file,$chunk);
				$ret=Model_Backup::addJobList($dbconnection,$jobs,$config['host_id']);
		
				$stats['backups']['present'] += $ret['error'];
				$stats['backups']['updated'] += $ret['updated'];
				$stats['backups']['added'] += $ret['added'];
			}
		}
	}
	$stats['time']['jobsaved'] = time();
	Model_BackupSettings::saveStats($dbconnection,$config['drv'],$config['host_id'],json_encode($stats));
	print_r($stats);
	}
	catch(Exception $ex){
		echo "Error : ".$ex->getMessage()."\n";
	}
	try{
		$ids = Model_Backup::getInProgressJobs($dbconnection,$config['host_id'],1);
		 echo "ids in progress : " . count($ids)."\n";
		 if(!empty($ids)){
        	foreach(array_chunk($ids,50) as $chunk){
                	$jobs = getJobsFields($gesasn,$script,$auth_file,$chunk);
                	$ret=Model_Backup::addJobList($dbconnection,$jobs,$config['host_id']);
        	}
		}
	}
	catch(Exception $ex){}
}
