--Tables de backups
CREATE TABLE backup_driver
(
  drv_id serial,
  drv_name character varying(20) NOT NULL,
  drv_type character varying(20) NOT NULL,
  drv_path character varying(150) NOT NULL,
  drv_admin_page character varying(150),
  CONSTRAINT drv_id_primary_key PRIMARY KEY (drv_id),
  CONSTRAINT drv_name_type_unique UNIQUE (drv_name, drv_type)
);

CREATE TABLE backup_config
(
  bck_cfg_id serial,
  bck_drv_id integer NOT NULL,
  bck_device_id integer NOT NULL,
  bck_params text,
  bck_stats text,
  bck_retention integer NOT NULL,
  CONSTRAINT "backup_config_PrimaryKey" PRIMARY KEY (bck_cfg_id),
  CONSTRAINT "backup_config_bck_drv_id_bck_device_id_unique" UNIQUE (bck_drv_id, bck_device_id),
  CONSTRAINT "backup_config_Foreign_key_backup_config_device_id" FOREIGN KEY (bck_device_id)
      REFERENCES esia_device (device_id) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT "backup_config_Foreign_key_backup_driver_drv_id" FOREIGN KEY (bck_drv_id)
      REFERENCES backup_driver (drv_id) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
);

CREATE TABLE backup_devicelink(
	bdev_id serial,
	bdev_config_id integer NOT NULL,
	bdev_type character varying(10) NOT NULL,
	bdev_name character varying(100) NOT NULL,
	bdev_data text,
	bdev_device_id integer,
	
	CONSTRAINT "backup_device_PrimaryKey" PRIMARY KEY (bdev_id),
	CONSTRAINT "bdev_name_unique" UNIQUE (bdev_name,bdev_type,bdev_config_id),
	CONSTRAINT "backup_device_Foreign_key_backup_config_bdev_config_id" FOREIGN KEY (bdev_config_id)
		REFERENCES esia_device (device_id) MATCH SIMPLE
      	ON UPDATE NO ACTION ON DELETE NO ACTION,
    CONSTRAINT "backup_device_Foreign_key_esia_device_device_id" FOREIGN KEY (bdev_device_id)
		REFERENCES esia_device (device_id) MATCH SIMPLE
      	ON UPDATE NO ACTION ON DELETE NO ACTION
);

CREATE TABLE backup_job(
	job_id serial,
	job_soft_id character varying(100) NOT NULL,
	job_name character varying(100) NOT NULL,
	job_server_id integer NOT NULL,
	job_source character varying(100),
	job_storage character varying(100),
	job_messages text,
	job_size bigint,
	job_type character varying(40),
	job_status integer NOT NULL,
	job_start_time timestamp NOT NULL, 
	job_end_time timestamp,
	CONSTRAINT job_id_primary_key PRIMARY KEY (job_id),
	CONSTRAINT job_srv_soft_id_unique UNIQUE (job_soft_id, job_server_id),
	CONSTRAINT "job_server_Foreign_key_bdev_device_id" FOREIGN KEY (job_server_id)
		REFERENCES esia_device (device_id)
);

--Insert into webpluginsi and advance config
INSERT INTO esia_web_plugin (web_name,web_dir_lang,web_index_page,web_admin_page,web_activate,web_plugin_apache_path,web_page_control) 
VALUES ('Backup','lang/',NULL,'bck_admin.php?id=:dev_id&action=:action',TRUE,'/plugin/backup/','bck_control.php?id=:id');

INSERT INTO esia_device_type (type_name,type_web_id)
VALUES ('backup server',(SELECT web_id FROM esia_web_plugin WHERE web_name = 'Backup'));

INSERT INTO esia_device_advance_config_type(device_config_type_name,device_config_type_controller,device_config_type_webplugin_id)
VALUES ('Backup','bck_admin.php',(SELECT web_id FROM esia_web_plugin WHERE web_name = 'Backup'));

--Ajout du plugin de backup
INSERT INTO esia_plugins (plugin_id,plugin_name,plugin_path,plugin_type,plugin_trap_oid,plugin_default_parameter,plugin_default_interval_time,plugin_description,plugin_delete,plugin_priority,plugin_html_template,plugin_tag) VALUES (DEFAULT,'CHECK_DB_BACKUPS','/usr/local/esia/plugins/check_backup.php','default',NULL,'-H $IP -c 0 -w 1 -p 48h -n <SRC_NAME>',300000,'Check if the X last backup of a OBJECT/SRC/NODE are success from a period of X(h|d|w)','false',7,NULL,'BACKUPS');

--Ajout des drivers de backups
INSERT INTO backup_driver (drv_name,drv_type,drv_path,drv_admin_page)
VALUES ('bareos','api','bareos-api.php','plugin/backup/admin/bareos-api.php');
INSERT INTO backup_driver (drv_name,drv_type,drv_path,drv_admin_page)
VALUES ('veeam','db','veeam-db.php','plugin/backup/admin/veeam-db.php');
INSERT INTO backup_driver (drv_name,drv_type,drv_path,drv_admin_page)
VALUES ('eurobackup','api','eurobackup-api.php','plugin/backup/admin/eurobackup-api.php');
